import Foundation

@available(iOS 11.0, *)
class RemoteNotificationsHandler : NSObject, UNUserNotificationCenterDelegate {
    
    let notificationOptions: UNAuthorizationOptions = [.badge, .sound, .alert]
    let notificationPresentationOptions: UNNotificationPresentationOptions = [.badge, .sound, .alert]
    let mainChannel: FlutterMethodChannel
    
    init(mainChannel: FlutterMethodChannel) {
        self.mainChannel = mainChannel
        super.init()
    }
    
    func userNotificationCenter(_ center: UNUserNotificationCenter,
                                willPresent notification: UNNotification,
                                withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        completionHandler(notificationPresentationOptions)
    }
    
    func userNotificationCenter(_ center: UNUserNotificationCenter,
                                didReceive response: UNNotificationResponse,
                                withCompletionHandler completionHandler: @escaping () -> Void) {
        guard response.actionIdentifier == UNNotificationDefaultActionIdentifier else {
            completionHandler()
            return
        }
        
        let content = response.notification.request.content.userInfo
        if let payload = content["aps"] as? [String: Any] {
            if let plugin = SwiftApplePushNotificationServicePlugin.instance {
                if plugin.mainChannelInitialized {
                    plugin.didSelectNotification(payload: payload)
                } else {
                    plugin.pendingExecuteEventQueue(payload: payload)
                }
            }
        }
        completionHandler()
    }
    
    func requestPermissionAndRegisterRemoteNotifications(application: UIApplication) {
        UNUserNotificationCenter.current().delegate = self
        UNUserNotificationCenter.current().requestAuthorization(options: notificationOptions) {
            granted, error in
            if error == nil && granted {
                DispatchQueue.main.async {
                    application.registerForRemoteNotifications()
                }
            }
        }
    }
    
    func didRegisterForRemoteNotificationsWithDeviceToken(deviceToken: Data) {
        let deviceTokenString = convertDeviceTokenToString(deviceToken: deviceToken)
        mainChannel.invokeMethod(NotificationMethodChannel.MethodCall.didRegisterForRemoteNotificationsWithDeviceToken, arguments: deviceTokenString)
    }
    
    private func convertDeviceTokenToString(deviceToken: Data) -> String {
        let tokenString = deviceToken.map {
            String(format: "%02.2hhx", $0)
        }.joined()
        return tokenString
    }
}

