import Flutter
import UIKit

@available(iOS 11.0, *)
public class SwiftApplePushNotificationServicePlugin: NSObject, FlutterPlugin {
    
    public static var instance: SwiftApplePushNotificationServicePlugin?
    public var mainChannelInitialized = false
    
    private var pendingPayload: [String: Any]?
    private var registrar: FlutterPluginRegistrar
    private var remoteNotificationHandler: RemoteNotificationsHandler
    private var mainChannel: FlutterMethodChannel

    private init(registrar: FlutterPluginRegistrar) {
        self.registrar = registrar
        mainChannel = FlutterMethodChannel(name: NotificationMethodChannel.Name.mainChannel, binaryMessenger: registrar.messenger())
        remoteNotificationHandler = RemoteNotificationsHandler(mainChannel: mainChannel)
        super.init()
        registrar.addMethodCallDelegate(self, channel: mainChannel)
    }

    public static func register(with registrar: FlutterPluginRegistrar) {
        objc_sync_enter(self)
        if instance == nil {
            instance = SwiftApplePushNotificationServicePlugin.init(registrar: registrar)
            registrar.addApplicationDelegate(instance!)
        }
        objc_sync_exit(self)
    }

    public func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [AnyHashable : Any] = [:]) -> Bool {
        remoteNotificationHandler.requestPermissionAndRegisterRemoteNotifications(application: application)
        return true
    }

    public func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        remoteNotificationHandler.didRegisterForRemoteNotificationsWithDeviceToken(deviceToken: deviceToken)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case NotificationMethodChannel.MethodCall.didInitMainChannel:
            mainChannelInitialized = true
            executeEventQueue()
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    private func executeEventQueue() {
        if let payload: [String: Any] = pendingPayload {
            didSelectNotification(payload: payload)
            pendingPayload = nil
        }
    }

    func pendingExecuteEventQueue(payload: [String: Any]) {
        pendingPayload = payload
    }
    
    func didSelectNotification(payload: [String: Any]) {
        mainChannel.invokeMethod(NotificationMethodChannel.MethodCall.didSelectNotification, arguments: payload)
    }
}
